@@ -399,6 +399,7 @@ struct Table {
   iterator end() { return iterator{*this, (int)length()}; }
 };
 
+
 struct Block {
   std::string name;
   std::vector<Item> items;
@@ -459,7 +460,7 @@ struct Block {
   Table find_mmcif_category(std::string cat);
 
   Loop& init_mmcif_loop(std::string cat, std::vector<std::string> tags) {
-    ensure_mmcif_category(cat);
+    ensure_mmcif_category(cat);  // modifies cat
     return setup_loop(find_mmcif_category(cat), cat, std::move(tags));
   }
 
@@ -470,6 +471,7 @@ private:
                    std::vector<std::string>&& tags);
 };
 
+
 struct Item {
   ItemType type;
   int line_number = -1;
@@ -508,10 +510,11 @@ struct Item {
     type = ItemType::Erased;
   }
 
+  // case-insensitive, the prefix should be lower-case
   bool has_prefix(const std::string& prefix) const {
-    return (type == ItemType::Pair && gemmi::starts_with(pair[0], prefix)) ||
+    return (type == ItemType::Pair && gemmi::istarts_with(pair[0], prefix)) ||
            (type == ItemType::Loop && !loop.tags.empty() &&
-            gemmi::starts_with(loop.tags[0], prefix));
+            gemmi::istarts_with(loop.tags[0], prefix));
   }
 
   void set_value(Item&& o) {
@@ -560,6 +563,45 @@ private:
   }
 };
 
+
+// ItemSpan is used to add tag-value pairs next to the same category tags.
+struct ItemSpan {
+  ItemSpan(std::vector<Item>& items)
+      : items_(items), begin_(0), end_(items.size()) {}
+  ItemSpan(std::vector<Item>& items, std::string prefix)
+      : ItemSpan(items) {
+    assert_tag(prefix);
+    prefix = gemmi::to_lower(prefix);
+    while (begin_ != end_ && !items_[begin_].has_prefix(prefix))
+      ++begin_;
+    if (begin_ != end_)
+      while (end_-1 != begin_ && !items_[end_-1].has_prefix(prefix))
+        --end_;
+  }
+  void set_pair(const std::string& tag, const std::string& value) {
+    assert_tag(tag);
+    std::string lctag = gemmi::to_lower(tag);
+    auto end = items_.begin() + end_;
+    for (auto i = items_.begin() + begin_; i != end; ++i) {
+      if (i->type == ItemType::Pair && gemmi::iequal(i->pair[0], lctag)) {
+        i->pair[0] = tag;  // if letter case differs, the tag changes
+        i->pair[1] = value;
+        return;
+      }
+      if (i->type == ItemType::Loop && i->loop.find_tag_lc(lctag) != -1) {
+        i->set_value(Item(tag, value));
+        return;
+      }
+    }
+    items_.emplace(end, tag, value);
+    ++end_;
+  }
+private:
+  std::vector<Item>& items_;
+  size_t begin_, end_;  // iterators would be invalidated by emplace()
+};
+
+
 inline void Loop::set_all_values(std::vector<std::vector<std::string>> columns){
   size_t w = columns.size();
   if (w != width())
@@ -728,22 +770,6 @@ inline const Pair* Block::find_pair(const std::string& tag) const {
   return item ? &item->pair : nullptr;
 }
 
-inline void Block::set_pair(const std::string& tag, const std::string& value) {
-  assert_tag(tag);
-  std::string lctag = gemmi::to_lower(tag);
-  for (Item& i : items) {
-    if (i.type == ItemType::Pair && gemmi::iequal(i.pair[0], lctag)) {
-      i.pair[1] = value;
-      return;
-    }
-    if (i.type == ItemType::Loop && i.loop.find_tag_lc(lctag) != -1) {
-      i.set_value(Item(tag, value));
-      return;
-    }
-  }
-  items.emplace_back(tag, value);
-}
-
 inline Column Block::find_loop(const std::string& tag) {
   Column c = find_values(tag);
   return c.item() && c.item()->type == ItemType::Loop ? c : Column();
@@ -799,6 +825,10 @@ inline size_t Block::get_index(const std::string& tag) const {
   fail(tag + " not found in block");
 }
 
+inline void Block::set_pair(const std::string& tag, const std::string& value) {
+  ItemSpan(items).set_pair(tag, value);
+}
+
 inline void Block::move_item(int old_pos, int new_pos) {
   if (old_pos < 0)
     old_pos += items.size();
@@ -932,6 +962,7 @@ inline Table Block::find_any(const std::string& prefix,
 
 inline Table Block::find_mmcif_category(std::string cat) {
   ensure_mmcif_category(cat);
+  cat = gemmi::to_lower(cat);
   std::vector<int> indices;
   for (Item& i : items)
     if (i.has_prefix(cat)) {
@@ -940,7 +971,7 @@ inline Table Block::find_mmcif_category(std::string cat) {
         for (size_t j = 0; j != indices.size(); ++j) {
           indices[j] = j;
           const std::string& tag = i.loop.tags[j];
-          if (!starts_with(tag, cat))
+          if (!istarts_with(tag, cat))
             fail("Tag ", tag, " in loop with ", cat);
         }
         return Table{&i, *this, indices, cat.length()};
