@@ -65,13 +65,19 @@ inline double it92_radius_approx(double b) {
   return (8.5 + 0.075 * b) / (2.4 + 0.0045 * b);
 }
 
-inline double get_minimum_b_iso(const Model& model) {
+inline double get_minimum_b(const Model& model) {
   double b_min = 1000.;
   for (const Chain& chain : model.chains)
     for (const Residue& residue : chain.residues)
-      for (const Atom& atom : residue.atoms)
-        if (atom.b_iso < b_min)
-          b_min = atom.b_iso;
+      for (const Atom& atom : residue.atoms) {
+        double b = atom.b_iso;
+        if (!atom.aniso.nonzero()) {
+          std::array<double,3> eig = atom.aniso.calculate_eigenvalues();
+          b = std::min(std::min(eig[0], eig[1]), eig[2]);
+        }
+        if (b < b_min)
+          b_min = b;
+      }
   return b_min;
 }
 
@@ -100,7 +106,7 @@ struct DensityCalculator {
     double spacing = requested_grid_spacing();
     if (spacing <= 0)
       spacing = grid.min_spacing();
-    double b_min = get_minimum_b_iso(model);
+    double b_min = get_minimum_b(model);
     blur = std::max(u_to_b() / 1.1 * sq(spacing) - b_min, 0.);
   }
 
