@@ -123,8 +123,6 @@ inline double cubic_interpolation_der(double u, double a, double b, double c, do
          + u * (4.5*b*u - 5*b + 1.5*d*u - d);
 }
 
-template<typename T, typename V=std::int8_t> struct MaskedGrid;
-
 // Order of grid axis. Some Grid functionality works only with the XYZ order.
 // The values XYZ and XYZ are used only when the grid covers whole unit cell.
 enum class AxisOrder : unsigned char {
@@ -147,6 +145,56 @@ struct GridMeta {
   Position get_position(int u, int v, int w) const {
     return unit_cell.orthogonalize(get_fractional(u, v, w));
   }
+
+  // operations re-scaled for faster later calculations; identity not included
+  std::vector<GridOp> get_scaled_ops_except_id() const {
+    std::vector<GridOp> grid_ops;
+    if (!spacegroup || spacegroup->number == 1)
+      return grid_ops;
+    if (axis_order != AxisOrder::XYZ)
+      fail("grid can use symmetries only if it is setup in the XYZ order");
+    GroupOps gops = spacegroup->operations();
+    grid_ops.reserve(gops.order());
+    for (const Op& so : gops.sym_ops)
+      for (const Op::Tran& co : gops.cen_ops) {
+        Op op = so.add_centering(co);
+        if (op != Op::identity()) {
+          // Rescale. Rotations are expected to be integral.
+          op.tran[0] = op.tran[0] * nu / Op::DEN;
+          op.tran[1] = op.tran[1] * nv / Op::DEN;
+          op.tran[2] = op.tran[2] * nw / Op::DEN;
+          for (int i = 0; i != 3; ++i)
+            for (int j = 0; j != 3; ++j)
+              op.rot[i][j] /= Op::DEN;
+          grid_ops.push_back({op});
+        }
+      }
+    return grid_ops;
+  }
+
+  // Quick but unsafe. assumes (for efficiency) that 0 <= u < nu, etc.
+  size_t index_q(int u, int v, int w) const {
+    return size_t(w * nv + v) * nu + u;
+  }
+
+  // Assumes (for efficiency) that -nu <= u < 2*nu, etc.
+  // Modifies arguments.
+  size_t index_n_ref(int& u, int& v, int& w) const {
+    if (u >= nu) u -= nu; else if (u < 0) u += nu;
+    if (v >= nv) v -= nv; else if (v < 0) v += nv;
+    if (w >= nw) w -= nw; else if (w < 0) w += nw;
+    return this->index_q(u, v, w);
+  }
+
+  // Assumes (for efficiency) that -nu <= u < 2*nu, etc.
+  size_t index_n(int u, int v, int w) const { return index_n_ref(u, v, w); }
+
+  // Assumes (for efficiency) that -nu <= u < nu, etc.
+  size_t index_near_zero(int u, int v, int w) const {
+    return this->index_q(u >= 0 ? u : u + nu,
+                         v >= 0 ? v : v + nv,
+                         w >= 0 ? w : w + nw);
+  }
 };
 
 template<typename T>
@@ -158,15 +206,16 @@ struct GridBase : GridMeta {
 
   std::vector<T> data;
 
+  void check_not_empty() const {
+    if (data.empty())
+      fail("grid is empty");
+  }
+
   void set_size_without_checking(int u, int v, int w) {
     nu = u, nv = v, nw = w;
     data.resize((size_t)u * v * w);
   }
 
-  // Quick but unsafe. assumes (for efficiency) that 0 <= u < nu, etc.
-  size_t index_q(int u, int v, int w) const {
-    return size_t(w * nv + v) * nu + u;
-  }
   T get_value_q(int u, int v, int w) const { return data[index_q(u, v, w)]; }
 
   size_t point_to_index(const Point& p) const { return p.value - data.data(); }
@@ -292,29 +341,9 @@ struct Grid : GridBase<T> {
     set_size_from_spacing(approx_spacing, denser);
   }
 
-  // Assumes (for efficiency) that -nu <= u < 2*nu, etc.
-  size_t index_n(int u, int v, int w) const {
-    if (u >= nu) u -= nu; else if (u < 0) u += nu;
-    if (v >= nv) v -= nv; else if (v < 0) v += nv;
-    if (w >= nw) w -= nw; else if (w < 0) w += nw;
-    return this->index_q(u, v, w);
-  }
-
-  // Assumes (for efficiency) that -nu <= u < nu, etc.
-  size_t index_near_zero(int u, int v, int w) const {
-    return this->index_q(u >= 0 ? u : u + nu,
-                         v >= 0 ? v : v + nv,
-                         w >= 0 ? w : w + nw);
-  }
-
-  void check_not_empty() const {
-    if (data.empty())
-      fail("grid is empty");
-  }
-
   // Safe but slower.
   size_t index_s(int u, int v, int w) const {
-    check_not_empty();
+    this->check_not_empty();
     return this->index_q(modulo(u, nu), modulo(v, nv), modulo(w, nw));
   }
 
@@ -323,7 +352,10 @@ struct Grid : GridBase<T> {
   }
 
   Point get_point(int u, int v, int w) {
-    return {u, v, w, &data[index_s(u, v, w)]};
+    u = modulo(u, nu);
+    v = modulo(v, nv);
+    w = modulo(w, nw);
+    return {u, v, w, &data[this->index_q(u, v, w)]};
   }
 
   Point get_nearest_point(const Fractional& f) {
@@ -351,7 +383,7 @@ struct Grid : GridBase<T> {
 
   // https://en.wikipedia.org/wiki/Trilinear_interpolation
   T interpolate_value(double x, double y, double z) const {
-    check_not_empty();
+    this->check_not_empty();
     int u, v, w;
     double xd = grid_modulo(x, nu, &u);
     double yd = grid_modulo(y, nv, &v);
@@ -429,7 +461,7 @@ struct Grid : GridBase<T> {
   }
   void copy_4x4x4(double& x, double& y, double& z,
                   std::array<std::array<std::array<T,4>,4>,4>& copy) const {
-    check_not_empty();
+    this->check_not_empty();
     auto prepare_indices = [this](double& r, int nt, int (&indices)[4]) {
       int t;
       r = this->grid_modulo(r, nt, &t);
@@ -454,7 +486,7 @@ struct Grid : GridBase<T> {
   }
 
   void get_subarray(T* dest, std::array<int,3> start, std::array<int,3> shape) const {
-    check_not_empty();
+    this->check_not_empty();
     if (this->axis_order != AxisOrder::XYZ)
       fail("get_subarray() is for Grids in XYZ order");
     const int u_start0 = modulo(start[0], nu);
@@ -480,7 +512,7 @@ struct Grid : GridBase<T> {
   }
 
   void set_subarray(const T* src, std::array<int,3> start, std::array<int,3> shape) {
-    check_not_empty();
+    this->check_not_empty();
     if (this->axis_order != AxisOrder::XYZ)
       fail("set_subarray() is for Grids in XYZ order");
     const int u_start0 = modulo(start[0], nu);
@@ -549,7 +581,7 @@ struct Grid : GridBase<T> {
         for (int u = u_lo; u <= u_hi; ++u) {
           Fractional fdelta = fctr - this->get_fractional(u, v, w);
           Position delta = unit_cell.orthogonalize_difference(fdelta);
-          size_t idx = UsePbc ? index_n(u, v, w) : this->index_q(u, v, w);
+          size_t idx = UsePbc ? this->index_n(u, v, w) : this->index_q(u, v, w);
           func(data[idx], delta);
         }
   }
@@ -590,32 +622,6 @@ struct Grid : GridBase<T> {
         d = new_value;
   }
 
-  // operations re-scaled for faster later calculations; identity not included
-  std::vector<GridOp> get_scaled_ops_except_id() const {
-    std::vector<GridOp> grid_ops;
-    if (!spacegroup || spacegroup->number == 1)
-      return grid_ops;
-    if (this->axis_order != AxisOrder::XYZ)
-      fail("grid can use symmetries only if it is setup in the XYZ order");
-    GroupOps gops = spacegroup->operations();
-    grid_ops.reserve(gops.order());
-    for (const Op& so : gops.sym_ops)
-      for (const Op::Tran& co : gops.cen_ops) {
-        Op op = so.add_centering(co);
-        if (op != Op::identity()) {
-          // Rescale. Rotations are expected to be integral.
-          op.tran[0] = op.tran[0] * nu / Op::DEN;
-          op.tran[1] = op.tran[1] * nv / Op::DEN;
-          op.tran[2] = op.tran[2] * nw / Op::DEN;
-          for (int i = 0; i != 3; ++i)
-            for (int j = 0; j != 3; ++j)
-              op.rot[i][j] /= Op::DEN;
-          grid_ops.push_back({op});
-        }
-      }
-    return grid_ops;
-  }
-
   template<typename Func>
   void symmetrize_using_ops(const std::vector<GridOp>& ops, Func func) {
     if (ops.empty())
@@ -631,7 +637,7 @@ struct Grid : GridBase<T> {
             continue;
           for (size_t k = 0; k < ops.size(); ++k) {
             std::array<int,3> t = ops[k].apply(u, v, w);
-            mates[k] = index_n(t[0], t[1], t[2]);
+            mates[k] = this->index_n(t[0], t[1], t[2]);
           }
           T value = data[idx];
           for (size_t k : mates) {
@@ -653,7 +659,7 @@ struct Grid : GridBase<T> {
   // grid point, then assign the result to all the points.
   template<typename Func>
   void symmetrize(Func func) {
-    symmetrize_using_ops(get_scaled_ops_except_id(), func);
+    symmetrize_using_ops(this->get_scaled_ops_except_id(), func);
   }
 
   // two most common symmetrize functions
@@ -671,66 +677,28 @@ struct Grid : GridBase<T> {
     symmetrize([](T a, T b) { return a + b; });
   }
 
-
-  template<typename V> std::vector<V> get_asu_mask() const {
-    std::vector<V> mask(data.size(), 0);
-    std::vector<GridOp> ops = get_scaled_ops_except_id();
-    size_t idx = 0;
-    for (int w = 0; w != nw; ++w)
-      for (int v = 0; v != nv; ++v)
-        for (int u = 0; u != nu; ++u, ++idx)
-          if (mask[idx] == 0)
-            for (const GridOp& op : ops) {
-              std::array<int, 3> t = op.apply(u, v, w);
-              size_t mate_idx = index_n(t[0], t[1], t[2]);
-              // grid point can be on special position
-              if (mate_idx != idx)
-                mask[mate_idx] = 1;
-            }
-    return mask;
+  T interpolate(const Fractional& f, int order) const {
+    switch (order) {
+      case 1: return *const_cast<Grid<T>*>(this)->get_nearest_point(f).value;
+      case 2: return interpolate_value(f);
+      case 3: return (T) tricubic_interpolation(f);
+    }
+    throw std::invalid_argument("interpolation \"order\" must 1, 2 or 3");
   }
 
-  MaskedGrid<T> masked_asu();
-};
-
-
-template<typename T, typename V> struct MaskedGrid {
-  std::vector<V> mask;
-  Grid<T>* grid;
-
-  struct iterator {
-    MaskedGrid& parent;
-    size_t index;
-    int u = 0, v = 0, w = 0;
-    iterator(MaskedGrid& parent_, size_t index_)
-      : parent(parent_), index(index_) {}
-    iterator& operator++() {
-      do {
-        ++index;
-        if (++u == parent.grid->nu) {
-          u = 0;
-          if (++v == parent.grid->nv) {
-            v = 0;
-            ++w;
-          }
+  // order 1=nearest, 2=linear, 3=cubic interpolation
+  void resample_to(Grid<T>& dest, int order) const {
+    dest.check_not_empty();
+    int idx = 0;
+    for (int w = 0; w < dest.nw; ++w)
+      for (int v = 0; v < dest.nv; ++v)
+        for (int u = 0; u < dest.nu; ++u, ++idx) {
+          const Fractional f = dest.get_fractional(u, v, w);
+          dest.data[idx] = interpolate(f, order);
         }
-      } while (index != parent.mask.size() && parent.mask[index] != 0);
-      return *this;
-    }
-    typename GridBase<T>::Point operator*() {
-      return {u, v, w, &parent.grid->data[index]};
-    }
-    bool operator==(const iterator &o) const { return index == o.index; }
-    bool operator!=(const iterator &o) const { return index != o.index; }
-  };
-  iterator begin() { return {*this, 0}; }
-  iterator end() { return {*this, mask.size()}; }
+  }
 };
 
-template<typename T> MaskedGrid<T> Grid<T>::masked_asu() {
-  return {get_asu_mask<std::int8_t>(), this};
-}
-
 
 template<typename T>
 Correlation calculate_correlation(const GridBase<T>& a, const GridBase<T>& b) {
