@@ -80,9 +80,10 @@ struct Vec3 {
   Vec3 normalized() const { return changed_magnitude(1.0); }
   double dist_sq(const Vec3& o) const { return (*this - o).length_sq(); }
   double dist(const Vec3& o) const { return std::sqrt(dist_sq(o)); }
-  double angle(const Vec3& o) const {
-    return std::acos(dot(o) / std::sqrt(length_sq() * o.length_sq()));
+  double cos_angle(const Vec3& o) const {
+    return dot(o) / std::sqrt(length_sq() * o.length_sq());
   }
+  double angle(const Vec3& o) const { return std::acos(cos_angle(o)); }
   bool approx(const Vec3& o, double epsilon) const {
     return std::fabs(x - o.x) <= epsilon &&
            std::fabs(y - o.y) <= epsilon &&
@@ -124,6 +125,17 @@ struct Mat33 {
     return Vec3(a[0][i], a[1][i], a[2][i]);
   }
 
+  Mat33 operator+(const Mat33& b) const {
+    return Mat33(a[0][0] + b[0][0], a[0][1] + b[0][1], a[0][2] + b[0][2],
+                 a[1][0] + b[1][0], a[1][1] + b[1][1], a[1][2] + b[1][2],
+                 a[2][0] + b[2][0], a[2][1] + b[2][1], a[2][2] + b[2][2]);
+  }
+  Mat33 operator-(const Mat33& b) const {
+    return Mat33(a[0][0] - b[0][0], a[0][1] - b[0][1], a[0][2] - b[0][2],
+                 a[1][0] - b[1][0], a[1][1] - b[1][1], a[1][2] - b[1][2],
+                 a[2][0] - b[2][0], a[2][1] - b[2][1], a[2][2] - b[2][2]);
+  }
+
   Vec3 multiply(const Vec3& p) const {
     return {a[0][0] * p.x + a[0][1] * p.y + a[0][2] * p.z,
             a[1][0] * p.x + a[1][1] * p.y + a[1][2] * p.z,
@@ -405,7 +417,7 @@ struct Correlation {
   double mean_y = 0.;
   void add_point(double x, double y) {
     ++n;
-    double weight = (n - 1.0) / n;
+    double weight = (double)(n - 1) / n;
     double dx = x - mean_x;
     double dy = y - mean_y;
     sum_xx += weight * dx * dx;
