@@ -188,6 +188,15 @@ void add_cif(py::module& cif) {
     .def("item_as_table", &Block::item_as_table)
     .def("get_index", &Block::get_index, py::arg("tag"))
     .def("set_pair", &Block::set_pair, py::arg("tag"), py::arg("value"))
+    .def("set_pairs",
+         [](Block &self, std::string prefix, py::dict data, bool raw) {
+           ItemSpan span(self.items, prefix);
+           for (auto item : data) {
+             std::string key = py::str(item.first);
+             std::string value = pyobject_to_string(item.second, raw);
+             span.set_pair(prefix + key, value);
+           }
+         }, py::arg("prefix"), py::arg("data"), py::arg("raw")=false)
     .def("init_loop", &Block::init_loop, py::arg("prefix"), py::arg("tags"),
          py::return_value_policy::reference_internal)
     .def("move_item", &Block::move_item, py::arg("old_pos"), py::arg("new_pos"))
@@ -212,10 +221,8 @@ void add_cif(py::module& cif) {
              if (values.back().size() != values[0].size())
                throw py::value_error("all columns must have equal length");
            }
-           if (w == 0 || values[0].size() == 0)
-             throw py::value_error("data cannot be empty");
            Loop& loop = self.init_mmcif_loop(std::move(name), std::move(tags));
-           loop.values.resize(w * values[0].size());
+           loop.values.resize(w != 0 ? w * values[0].size() : 0);
            for (size_t col = 0; col != w; ++col) {
              size_t idx = col;
              for (auto handle : values[col]) {
