@@ -8,7 +8,7 @@
 #include "gemmi/seqid.hpp"
 #include "gemmi/metadata.hpp"
 #include "gemmi/enumstr.hpp"
-#include "gemmi/tostr.hpp"
+#include "tostr.hpp"
 #include "meta.h"
 
 namespace py = pybind11;
@@ -39,6 +39,8 @@ void add_meta(py::module& m) {
     .def("__repr__", [](const SeqId& self) {
         return "<gemmi.SeqId " + self.str() + ">";
     })
+    .def(py::self == py::self)
+    .def(py::self < py::self)
     .def(py::pickle(
         [](const SeqId &self) {
             return py::make_tuple(self.num, self.icode);
@@ -59,6 +61,7 @@ void add_meta(py::module& m) {
     .def("__repr__", [](const ResidueId& self) {
         return "<gemmi.ResidueId " + self.str() + ">";
     })
+    .def(py::self == py::self)
     .def(py::pickle(
         [](const ResidueId &self) {
             return py::make_tuple(self.seqid, self.segment, self.name);
@@ -89,6 +92,7 @@ void add_meta(py::module& m) {
     .def("__repr__", [](const AtomAddress& self) {
         return tostr("<gemmi.AtomAddress ", self.str(), '>');
     })
+    .def(py::self == py::self)
     .def(py::pickle(
         [](const AtomAddress &self) {
             return py::make_tuple(self.chain_name, self.res_id, self.atom_name, self.altloc);
@@ -244,4 +248,8 @@ void add_meta(py::module& m) {
     .def_readwrite("special_kind", &Assembly::special_kind)
     ;
   py::bind_vector<std::vector<Assembly>>(m, "AssemblyList");
+
+  py::class_<Metadata>(m, "Metadata")
+    .def_readwrite("authors", &Metadata::authors)
+    ;
 }
