@@ -35,6 +35,11 @@ template<typename P, typename C> C& get_child(P& parent, int index) {
   return children[normalize_index(index, children)];
 }
 
+template<typename P, typename C> void set_child(P& parent, int index, C& child) {
+  auto& children = parent.children();
+  children[normalize_index(index, children)] = child;
+}
+
 template<typename P> void remove_child(P& parent, int index) {
   auto& children = parent.children();
   children.erase(children.begin() + normalize_index(index, children));
@@ -113,6 +118,7 @@ void add_mol(py::module& m) {
     .def("__delitem__", &remove_child<Structure>, py::arg("index"))
     .def("__delitem__", &remove_children<Structure>)
     .def("__delitem__", &Structure::remove_model, py::arg("name"))
+    .def("__setitem__", &set_child<Structure, Model>)
     .def("find_connection", &Structure::find_connection,
          py::arg("partner1"), py::arg("partner2"),
          py::return_value_policy::reference_internal)
@@ -140,6 +146,10 @@ void add_mol(py::module& m) {
     .def("remove_ligands_and_waters", remove_ligands_and_waters<Structure>)
     .def("shorten_chain_names", &shorten_chain_names)
     .def("expand_ncs", &expand_ncs, py::arg("how"))
+    .def("transform_to_assembly",
+         [](Structure& st, const std::string& assembly_name, HowToNameCopiedChain how) {
+        return transform_to_assembly(st, assembly_name, how, nullptr);
+    }, py::arg("assembly_name"), py::arg("how"))
     .def("calculate_box", &calculate_box, py::arg("margin")=0.)
     .def("calculate_fractional_box", &calculate_fractional_box, py::arg("margin")=0.)
     .def("clone", [](const Structure& self) { return new Structure(self); })
