@@ -3,6 +3,7 @@
 #include "gemmi/unitcell.hpp"
 #include "tostr.hpp"  // tostr
 #include "gemmi/cellred.hpp"  // GruberVector
+#include "gemmi/twin.hpp"  // find_lattice_2fold_ops
 
 #include <cstdio>  // for snprintf
 #include <array>
@@ -145,6 +146,8 @@ void add_unitcell(py::module& m) {
     })
     .def("row_copy", &Mat33::row_copy)
     .def("column_copy", &Mat33::column_copy)
+    .def(py::self + py::self)
+    .def(py::self - py::self)
     .def("multiply", (Mat33 (Mat33::*)(const Mat33&) const) &Mat33::multiply)
     .def("multiply", (Vec3 (Mat33::*)(const Vec3&) const) &Mat33::multiply)
     .def("left_multiply", &Mat33::left_multiply)
@@ -263,6 +266,9 @@ void add_unitcell(py::module& m) {
     .def_readonly("volume", &UnitCell::volume)
     .def_readonly("images", &UnitCell::images)
     .def_property_readonly("parameters", &make_parameters_tuple)
+    .def_readonly("frac", &UnitCell::frac)
+    .def_readonly("orth", &UnitCell::orth)
+    // These two functions are deprecated, use frac.mat and orth.mat.
     .def_property_readonly("fractionalization_matrix",
                            [](const UnitCell& self) { return self.frac.mat; })
     .def_property_readonly("orthogonalization_matrix",
@@ -276,6 +282,8 @@ void add_unitcell(py::module& m) {
          py::arg("sg"), py::arg("eps")=1e-3)
     .def("is_crystal", &UnitCell::is_crystal)
     .def("approx", &UnitCell::approx, py::arg("other"), py::arg("epsilon"))
+    .def("is_similar", &UnitCell::is_similar,
+         py::arg("other"), py::arg("rel"), py::arg("deg"))
     .def("calculate_u_eq", &UnitCell::calculate_u_eq)
     .def("fractionalize", &UnitCell::fractionalize)
     .def("orthogonalize", &UnitCell::orthogonalize)
@@ -330,10 +338,10 @@ void add_unitcell(py::module& m) {
 
   py::class_<GruberVector>(m, "GruberVector")
     .def(py::init<const std::array<double,6>&>())
-    .def(py::init([](const UnitCell& u, const SpaceGroup* sg) {
-        Mat33 m = u.primitive_orth_matrix(sg ? sg->centring_type() : 'P');
-        return new GruberVector(m);
-    }))
+    .def(py::init<const UnitCell&, const SpaceGroup*, bool>(),
+         py::arg("cell"), py::arg("sg"), py::arg("track_change_of_basis")=false)
+    .def(py::init<const UnitCell&, char, bool>(),
+         py::arg("cell"), py::arg("centring"), py::arg("track_change_of_basis")=false)
     .def_property_readonly("parameters", [](const GruberVector& g) {
       return make_six_tuple(g.parameters());
     })
@@ -341,7 +349,10 @@ void add_unitcell(py::module& m) {
       return make_six_tuple(self.cell_parameters());
     })
     .def("get_cell",
-         [](const GruberVector& self) { return new UnitCell(self.cell_parameters()); })
+         [](const GruberVector& self) { return new UnitCell(self.get_cell()); })
+    .def_property_readonly("change_of_basis", [](const GruberVector& self) {
+        return self.change_of_basis.get();
+    }, py::return_value_policy::reference_internal)
     .def("selling", &GruberVector::selling)
     .def("is_normalized", &GruberVector::is_normalized)
     .def("is_buerger", &GruberVector::is_buerger, py::arg("epsilon")=1e-9)
@@ -361,10 +372,7 @@ void add_unitcell(py::module& m) {
 
   selling_vector
     .def(py::init<const std::array<double,6>&>())
-    .def(py::init([](const UnitCell& u, const SpaceGroup* sg) {
-        Mat33 m = u.primitive_orth_matrix(sg ? sg->centring_type() : 'P');
-        return new SellingVector(m);
-    }))
+    .def(py::init<const UnitCell&, const SpaceGroup*>())
     .def_property_readonly("parameters", [](const SellingVector& self) {
       return make_six_tuple(self.s);
     })
@@ -372,7 +380,7 @@ void add_unitcell(py::module& m) {
       return make_six_tuple(self.cell_parameters());
     })
     .def("get_cell",
-         [](const SellingVector& self) { return new UnitCell(self.cell_parameters()); })
+         [](const SellingVector& self) { return new UnitCell(self.get_cell()); })
     .def("sum_b_squared", &SellingVector::sum_b_squared)
     .def("gruber", &SellingVector::gruber)
     .def("is_reduced", &SellingVector::is_reduced, py::arg("epsilon")=1e-9)
@@ -387,4 +395,12 @@ void add_unitcell(py::module& m) {
                  self.s[0], self.s[1], self.s[2], self.s[3], self.s[4], self.s[5]);
         return std::string(buf);
     });
+
+  m.def("find_lattice_2fold_ops", &find_lattice_2fold_ops,
+        py::arg("reduced_cell"), py::arg("max_obliq"));
+  m.def("find_lattice_symmetry_r", &find_lattice_symmetry_r);
+  m.def("find_lattice_symmetry", &find_lattice_symmetry,
+        py::arg("cell"), py::arg("centring"), py::arg("max_obliq"));
+  m.def("find_twin_laws", &find_twin_laws,
+        py::arg("cell"), py::arg("sg"), py::arg("max_obliq"), py::arg("all_ops"));
 }
